# Copyright (C) 2009-2011, Ecole Polytechnique Federale de Lausanne (EPFL) and
# Hospital Center and University of Lausanne (UNIL-CHUV), Switzerland
# All rights reserved.
#
#  This software is distributed under the open-source license Modified BSD.

""" Defines the graphical user interface to the Connectome Mapper
"""
import os, os.path as op
import sys

try:
	from traits.api import HasTraits, Int, Str, Directory, List,\
	                 Bool, File, Button, Enum, Instance
	from traits.ui.api import View, Item, HGroup, Handler, HFlow, \
	                    message, spring, Group, VGroup, TableEditor, UIInfo
	from traits.ui.menu import OKButton, Action
except ImportError:
	try:
		from enthought.traits.api import HasTraits, Int, Str, Directory, List,\
		                 Bool, File, Button, Enum, Instance
		from enthought.traits.ui.api import View, Item, HGroup, Handler, HFlow, \
		                    message, spring, Group, VGroup, TableEditor, UIInfo
		from enthought.traits.ui.menu import OKButton, Action
	except ImportError:
		sys.exit('Error: could not load Enthought Traits module. Exiting...')

  
from configuration import PipelineConfiguration

from helpgui import HelpDialog, AboutDialog
 
try:
	import rpy2.robjects as robjects
except ImportError:
	sys.exit('Error: could not load rpy2 module. Make sure that: \n\t- rpy2 is correctly installed\n\t- the LD_LIBRARY_PATH and R_HOME environmental variables are set to the appropriate paths')
	

from info import __version__

class MyHandler(Handler):
    def exit_cma(self,info):
	a = Handler.close(self, info, False)
	if a:
		info.ui.dispose()
		return

    def about_cma(self,info):
	a = AboutDialog()
	a.configure_traits(kind='livemodal')

    def help_cma(self,info):
	b = HelpDialog()
	b.configure_traits(kind='livemodal')

    def _Save_button_fired(self,info):
    	import pickle
        import enthought.sweet_pickle as sp
        import os.path
        from enthought.pyface.api import FileDialog, OK
        
        wildcard = "CMA Configuration State (*.pkl)|*.pkl|" \
                        "All files (*.*)|*.*"
        dlg = FileDialog(wildcard=wildcard,title="Filename to store configuration state",\
                         resizeable=False, action = 'save as', \
                         default_directory=info.object.study_dir)
        
        if dlg.open() == OK:
            if not dlg.path.endswith('.pkl'):
                dlg.path = dlg.path + '.pkl'
            info.object.save_state(dlg.path)

    def _Load_button_fired(self,info):
	import enthought.sweet_pickle as sp
        from enthought.pyface.api import FileDialog, OK
        
        wildcard = "CMA Configuration State (*.pkl)|*.pkl|" \
                        "All files (*.*)|*.*"
        dlg = FileDialog(wildcard=wildcard,title="Select a configuration state to load",\
                         resizeable=False, \
                         default_directory=info.object.study_dir)
        
        if dlg.open() == OK:            
            if not os.path.isfile(dlg.path):
                return
            else:
                info.object.load_state(dlg.path)

class CMAGUI( PipelineConfiguration ):
    """ The Graphical User Interface for the CMA
    """
        
    run_global_analysis = Button
    run_edges_analysis = Button
    run_nodes_analysis = Button
    run_SNWA_analysis = Button
    run_nbs = Button
    run_relaxation = Button
    run_single_subject = Button
    Save_button = Action(name='Save',action='_Save_button_fired')
    Load_button = Action(name='Load',action='_Load_button_fired')
    Exit_button = Action(name='Exit',action='exit_cma')
    About_button = Action(name='About',action='about_cma')
    CMA_help_button = Action(name='Help ',action='help_cma')

#    group_directories = VGroup(Item('group1_dir', label='Group1 Directory:', tooltip = 'Directory containing group1 connectivity matrices'),
#			       Item('group2_dir', label='Group2 Directory:', tooltip = 'Directory containing group2 connectivity matrices'),
#			       label = "Data")

    group_directories = VGroup(Item('study_dir', label='Study Directory:', tooltip = 'Directory containing group1 and group 2 connectivity folders '),
			       HGroup(Item('data_separ', label='Data separator:',tooltip='Character that separates columns in the input data files'),spring),
			       label = "Data")

    multivariate_directories = VGroup(Item('multivariate_dirs', label='Directories:', tooltip = 'Specify a directory for each modality',style='custom'),
			       HGroup(Item('n_perm',label = 'Permutations:', tooltip = 'Number of permutations to estimate the null distribution',visible_when = '(len(multivariate_dirs)>1) and (test_type == \'non-parametric\')'),
				      Item('data_separ', label='Data separator:',tooltip='Character that separates columns in the input data files'),spring),
			       label = "Data")

    mask_group = HGroup(
			Item('masking',label='Mask data:',tooltip = 'Check if you want to mask the connectome matrices using a file'),
			Item('mask_path',label='Mask file:', tooltip = 'Select the file to use as a mask',visible_when='masking'),
			Item('mask_separ',label='Separator:',tooltip = 'Character that separates columns in the mask file',visible_when='masking')
			)

    plot_results_group = VGroup(Item('plot_results',label="Plot results ?"),
		      Item('node_labels',label = "Node labels:",visible_when="plot_results"),
		      HGroup(Item('nodes_color', label="Node's color:"),Item('edges_color',label = "Edge's color:"),visible_when="plot_results"))

    decomposition_item = HGroup(Item('decomposition_method', label='Decomposition method:', tooltip = 'Select "file" if you have a predefined decomposition, "automatic" if you want to automaticaly decompose the connectivit matrices according to the Group1 mean connectivity matrix'),
				VGroup(Item('decomposition_file',label='Decomposition file:',tooltip = 'Select the file containing your node decomposition'),
				       Item('decomposition_separ',label='Separator:',tooltip='Character that separates columns in the decomposition file'),visible_when='decomposition_method==\'File\''),
				Item('automatic_decompositions',label='Algorithm:',tooltip = 'Select the algorithm for the automatic decomposition',visible_when='decomposition_method==\'Automatic\''))

    one_step = VGroup(decomposition_item,
		      Item('MCPProcedure', label='Correction:', tooltip = 'Correction done after each step'),
		      Item('test_type',label='Test type:',tooltip = 'Parametric or non-parametric testing'),
		      Item('paired',label='Pairing:',tooltip = 'Performs a paired test : make sure the subject pairs have the same rank in alphabetical order inside the two groups'),
		      Item('alternative',label='Alternative:',tooltip='Alternative to the null hypothesis')
		      )

    two_steps = VGroup(one_step,
		       # Item('RM',label = "Relaxation Method", tooltip = "Relaxation Method as defined by Meskaldji (2013)"),
		       HGroup(Item('alpha1', label='Alpha 1:', tooltip = 'Significant threshold after 1st step'),
			      Item('alpha2', label='Alpha 2:', tooltip = 'Significant threshold after relaxation'),
			      #Item('strong',label='Strong',tooltip="Factor to apply to edges outside significant clusters", visible_when = '(RM == \'RMIO\') or (RM == \'All\')')
			      Item('strong',label='Strong',tooltip="Factor to apply to edges outside significant clusters")
			     ),
		       label = "Relaxation parameters")

    network_measures =   VGroup( VGroup(Item('binarize',label='Binarize matrices', tooltip='Binarizes the matrices before computing network measures'),
                                  Item('is_weighted',label='Weighted matrices:',tooltip='Select if values in the connectivity matrices represent weights and not number of connections between regions'),show_border = True),
				 HFlow( 
			  	  Item('mean_degree',label='Mean Degree', tooltip='Network mean degree'),
				  Item('mean_betweenness',label='Betweenness',tooltip='Network mean betweenness'),
			  	  Item('mean_closeness',label='Closeness', tooltip='Network mean closeness'),
				  Item('mean_diameter',label = 'Diameter',tooltip = 'Network mean diameter'),
				  Item('global_efficiency',label='Efficiency',tooltip='Network global efficiency'),
				  #Item('small_worldness',label='Small-worldness',tooltip='Network small-worldness compared to density equivalent random graph'),
				  show_labels = True),
			   label = "Network measures")

    subnetwork_measures =   VGroup( VGroup(Item('binarize',label='Binarize matrices', tooltip='Binarizes the matrices before computing network measures'),
                                  Item('is_weighted',label='Weighted matrices:',tooltip='Select if values in the connectivity matrices represent weights and not number of connections between regions'),show_border = True),
				HFlow( 
			  	  Item('mean_degree',label='Mean Degree', tooltip='Network mean degree'),
				  Item('mean_betweenness',label='Betweenness',tooltip='Network mean betweenness'),
			  	  Item('mean_closeness',label='Closeness', tooltip='Network mean closeness'),
				  Item('mean_diameter',label = 'Diameter',tooltip = 'Network mean diameter'),
				  Item('global_efficiency',label='Efficiency',tooltip='Network global efficiency'),show_labels = True),
			   label = "Subnetwork measures")

    node_measures = VGroup(VGroup(Item('binarize',label='Binarize matrices', tooltip='Binarizes the matrices before computing network measures'),
                                  Item('is_weighted',label='Weighted matrices:',tooltip='Select if values in the connectivity matrices represent weights and not number of connections between regions'),show_border = True),
			       HFlow(
			  	Item('degree',label="Degree",tooltip='Nodal degree'),
				Item('betweenness',label="Betweenness",tooltip='Nodal betweenness'),
				Item('closeness',label='Closeness',tooltip="Nodal closeness"),
				Item('diameter',label='Diameter',tooltip="Nodal diameter"),show_labels=True
				),
			  label = 'Node Measures')


    regress_group = HGroup(Item('regress',label = "Regress data:", tooltip = 'Check to regress data from the input'),
		           Item('regression_file',label = "Regression file:",tooltip='Select the file containing the regressors', visible_when='regress'),
			   Item('regress_separ',label='Separator:',tooltip='Character that separates columns in the regression file',visible_when = "regress"))

    global_study = VGroup(
		    group_directories,
		    regress_group,
		    network_measures,
		    Group(Item('alternative',label='Alternative:',tooltip='Alternative to the null hypothesis'),
			  Item('test_type',label='Test type:',tooltip = 'Parametric or non-parametric testing'),label="Test parameters"),
		    HGroup(spring,Item('run_global_analysis',label='Run global analysis',show_label = False),springy=True),
		    label = "Global Analysis",
		    show_border = False
		    )

    SNWA_study = VGroup(
		    group_directories,
		    regress_group,
		    subnetwork_measures,
		    VGroup(one_step,label = "SNWA parameters"),
		    Item('alpha1', label='Alpha:', tooltip = 'Significant threshold'),
		    plot_results_group,
		    HGroup(spring,Item('run_SNWA_analysis',label='Run SNWA analysis',show_label = False)),
		    label = "SNWA Analysis",
		    show_border = False
		    )	

    NBS_analysis = VGroup(
		    group_directories,
		    mask_group,
		    VGroup(
			Item('NBS_thresh',label = 'Subnetwork forming threshold',tooltip='T value above which the connections will be used to find connected components'),
			Item('NBS_permutations',label='Permutations', tooltip='Number of permutation to estimate the null distribution'),
			Item('NBS_tail', label='Tail',tooltip='Tail of the test. "Both" tests against H1 = groups have different means, "left" tests against H1 = Group1 > Group2 and "right" tests against H1 = Group1 < Group2'),
			label = 'NBS parameters'),
		    HGroup(spring,Item('run_nbs',label='Run NBS analysis',show_label=False)),
		    label = 'NBS Analysis',
		    show_border = False
		    )	    

    edges_study = VGroup(
		    multivariate_directories,
		    mask_group,
                    regress_group,
		    Group(Item('summary_function',label='Summary Function:', tooltip = 'Subnetwork summary function'),label="Subnetwork summary function"),
		    two_steps,
		    plot_results_group,		    
		    HGroup(spring,Item('run_edges_analysis',label="Run connections analysis",show_label = False)),		    
                    label = "Connections Analysis",
                    show_border = False
                    )

    nodes_study = VGroup(
		    group_directories,
		    regress_group,
		    node_measures,
		    two_steps,
		    HGroup(spring,Item('run_nodes_analysis',label="Run nodes analysis",show_label = False)),		    
                    label = "Nodes Analysis",
                    show_border = False
                    )


    matrix_relaxation = Group(
                    VGroup(
		    VGroup(Item('matrix_to_relax', label='Matrix file:', tooltip = 'Please select the matrix of p-values to relax'),Item('data_separ',label='Data separator:',tooltip='Character that separates columns in the matrix file'),label="Data"),
		    decomposition_item,
		    Item('MCPProcedure', label='Correction:', tooltip = 'Correction done after each step'),
		    HGroup(Item('alpha1', label='Alpha 1:', tooltip = 'Significant threshold after 1st step'),
			      Item('alpha2', label='Alpha 2:', tooltip = 'Significant threshold after relaxation'),
			      #Item('strong',label='Strong',tooltip="Factor to apply to edges outside significant clusters", visible_when = '(RM == \'RMIO\') or (RM == \'All\')')
			      Item('strong',label='Strong',tooltip="Factor to apply to edges outside significant clusters")
			     ),
		    HGroup(spring,Item('run_relaxation',label="Run matrix relaxation",show_label = False))
                    ),	
                    label = "Matrix relaxation",
                    show_border = False
                    )

    single_subject = Group(
		    VGroup(
		    Item('single_matrix',label='Connectivity Matrix:',tooltip="Select a single subject connectivity matrix"),Item('data_separ',label='Data separator:',tooltip='Character that separates columns in the matrix file'),label="Data"),
		    network_measures,
		    HGroup(spring,Item('run_single_subject',label='Run single subject analysis',show_label=False)),
		    label="Single subject analysis",
		    show_border=False
		    )
        
    view = View(
            HGroup(
	      global_study,
	      SNWA_study,
	      NBS_analysis,
	      nodes_study,
              edges_study,
              matrix_relaxation,
	      single_subject,
              orientation= 'horizontal',
              layout='tabbed',
              springy=True
            ),
        resizable = True,
        width=0.3,
        title     = ('Connectome Analyzer v' + __version__),
	handler = MyHandler,
	buttons   = [ About_button, CMA_help_button, Save_button,Load_button,Exit_button ]
    )

    def check_file(self,filepath):
	if not(op.isfile(filepath)):
		print('ERROR: File ' + filepath + ' does not exist. Operation stopped.')
		message(message = 'File ' + filepath + ' does not exist. Operation stopped', title = 'Error', buttons = [ 'OK' ], parent = None)
		return False
	return True

    def check_groups(self,study_dir):
	if not(op.exists(op.join(study_dir,'Group1')) and op.exists(op.join(study_dir,'Group2'))):
		print('ERROR: The study folder "' + study_dir + '" is not valid. Please enter the path to a folder containing "Group1" and "Group2".')
		message(message = 'The study folder is not valid. Please enter the path to a folder containing "Group1" and "Group2".', title = 'Error', buttons = [ 'OK' ], parent = None)	
		print('[  OPERATION ABORTED  ]')	
		return False
	return True

    def check_data(self, check_regress,check_paired,*dir_to_check):
	group1_files = []
	group2_files = []
	extensions = []
	for d in dir_to_check:
		d = d.encode('ascii')
		if not self.check_groups(d):
			return False
		group1_files = group1_files + (os.listdir(op.join(d,'Group1')))
		group2_files = group2_files + (os.listdir(op.join(d,'Group2')))
	for f in group1_files + group2_files:
		extensions.append(op.splitext(f)[1])
	extensions = list(set(extensions))
	if len(extensions)>1:
		print('WARNING: The dataset contains files that are in different formats (' + ', '.join(extensions) + ')')	
	if (check_regress):
		if not(self.check_file(self.regression_file)):
			print('[  OPERATION ABORTED  ]')
			return False
	if (check_paired):
		if len(group1_files)!=len(group2_files):
			print('ERROR: The two groups have different number of files. A paired test cannot be done.')
			message(message = 'The two groups have different number of files. A paired test cannot be done.', title = 'Error', buttons = [ 'OK' ], parent = None)	
			print('[  OPERATION ABORTED  ]')
			return False
		print('WARNING: You are performing a paired test. Make sure the paired files have the same rank in alphabetical order in Group1 and Group2')
	return True

    def check_measures(self,selected_summary_statistics):
	if sum(selected_summary_statistics) < 1:		
		print('ERROR: Please select at least one network measure')
		message(message = 'Please select at least one measure', title = 'Error', buttons = [ 'OK' ], parent = None)
		print('[  OPERATION ABORTED  ]')
		return False
	return True

    def _run_global_analysis_fired(self):
	print('[  GLOBAL ANALYSIS  ]')
	selected_summary_statistics = [self.mean_degree, self.mean_betweenness, self.mean_closeness, self.mean_diameter, self.global_efficiency]	# self.small_worldness
	if self.check_data(self.regress,self.paired,self.study_dir) and self.check_measures(selected_summary_statistics):
		robjects.globalenv['connectomeAnalyzerPath'] = os.getenv('ConnectomeAnalyzerPath')
		robjects.r('''
			source(\'''' + op.join(os.getenv('ConnectomeAnalyzerPath'),'GlobalVectorAnalysis.r') + '''\')''')
		python_GlobalVectorAnalysis = robjects.globalenv['GlobalVectorAnalysis']
		summary_statistics_names = ['MeanDegree','MeanBetweenness','MeanCloseness','MeanDiameter','GlobalEfficiency'] 	# 'SmallWorldness'
		for i in range(0,len(selected_summary_statistics)):
			if selected_summary_statistics[i]:
				python_GlobalVectorAnalysis(self.study_dir,self.data_separ,False,self.mask_path,self.mask_separ,self.regress_separ,summary_statistics_names[i], self.binarize, self.is_weighted, self.test_type,self.alternative, self.regression_file, self.regress, self.paired)
		print('[  DONE  ]')

    def _run_SNWA_analysis_fired(self):
	print('[  SNWA ANALYSIS  ]')
	selected_summary_statistics = [self.mean_degree, self.mean_betweenness, self.mean_closeness, self.mean_diameter, self.global_efficiency]
	summary_statistics_names = ['MeanDegree','MeanBetweenness','MeanCloseness','MeanDiameter','GlobalEfficiency']
	if self.check_data(self.regress,self.paired,self.study_dir) and self.check_measures(selected_summary_statistics):
		robjects.globalenv['connectomeAnalyzerPath'] = os.getenv('ConnectomeAnalyzerPath')
		robjects.r('''
			source(\'''' + op.join(os.getenv('ConnectomeAnalyzerPath'),'SNWA.r') + '''\')''')
		python_SNWA_analysis = robjects.globalenv['SNWA']
		
		for i in range(0,len(selected_summary_statistics)):
			if selected_summary_statistics[i]:
				python_SNWA_analysis(self.study_dir, self.data_separ,False,self.mask_path,self.mask_separ,self.regress_separ,self.decomposition_separ,self.decomposition_method, self.automatic_decompositions, self.decomposition_file, self.alpha1, self.MCPProcedure,self.test_type, self.n_perm, self.paired, summary_statistics_names[i], self.binarize, self.is_weighted, self.alternative, self.plot_results, self.regression_file, self.regress)
		print('[  DONE  ]')

    def _run_nbs_fired(self):
	try:
		import numpy as np
	
		from nbs import compute_nbs

		print('[  NBS ANALYSIS  ]')
		if self.check_data(False,False,self.study_dir):
			gp1_files = os.listdir(op.join(self.study_dir,'Group1'))
			gp2_files = os.listdir(op.join(self.study_dir,'Group2'))
			temp_ext = op.splitext(op.join(self.study_dir,'Group1',gp1_files[0]))[1]
			if temp_ext == '.mat':
				try:
					import scipy.io
					temp = scipy.io.loadmat(op.join(self.study_dir,'Group1',gp1_files[0]))
					temp = temp['matrix']
					N1,N2 = temp.shape
				except ImportError:
					print('Error: Could not import scipy module. Please install Scipy to read mat files')
			else:
				N1,N2 = np.shape(np.genfromtxt(op.join(self.study_dir,'Group1',gp1_files[0]),delimiter=self.data_separ))
			if N1!=N2:
				print('Error: file ' + gp1_files[0] + ' has dimensions ' + N1 + 'x' + N2 + ' and is not square')
				return
			if self.masking:
				mask_ext = op.splitext(self.mask_path)[1]
				if mask_ext == '.mat':
					try:
						import scipy.io
						temp = scipy.io.loadmat(self.mask_path)
						mask = temp['matrix']
						if mask.shape != (N1,N2):
							print('Error: mask ' + self.mask_path + ' do not match the dimensions of ' + p.join(self.study_dir,'Group1',gp1_files[0]))
							return
					except ImportError:
						print('Error: Could not import scipy module. Please install Scipy to read mat files')
				else:
					mask = np.genfromtxt(self.mask_path,delimiter=self.mask_separ)
					if np.shape(mask) != (N1,N2):
						print('Error: mask ' + self.mask_path + ' do not match the dimensions of ' + p.join(self.study_dir,'Group1',gp1_files[0]))
						return
			X = np.zeros((N1,N2,len(gp1_files)))
			for f in range(0,len(gp1_files)):
				temp_file = op.join(self.study_dir,'Group1',gp1_files[f])
				temp_ext = op.splitext(temp_file)[1]
				if temp_ext == '.mat':
					try:
						import scipy.io
						temp = scipy.io.loadmat(temp_file)
						temp = temp['matrix']
					except ImportError:
						print('Error: Could not import scipy module. Please install Scipy to read mat files')
				else:
					temp = np.genfromtxt(temp_file,delimiter=self.data_separ)
				temp_N1, temp_N2 = temp.shape
				if (temp_N1,temp_N2) != (N1,N2):
					print('Error: file ' + gp1_files[f] + ' has dimension ' + temp_N1 + 'x' + temp_N2 + ' and does not match the one from file ' + gp1_files[0])
					return
				if(self.masking):
					temp[mask == 0] = 0
				X[:,:,f] = temp
			Y = np.zeros((N1,N2,len(gp2_files)))
			for f in range(0,len(gp2_files)):
				temp_file = op.join(self.study_dir,'Group2',gp2_files[f])
				temp_ext = op.splitext(temp_file)[1]
				if temp_ext == '.mat':
					try:
						import scipy.io
						temp = scipy.io.loadmat(temp_file)
						temp = temp['matrix']
					except ImportError:
						print('Error: Could not import scipy module. Please install Scipy to read mat files')
				else:
					temp = np.genfromtxt(temp_file,delimiter=self.data_separ)
				temp_N1, temp_N2 = temp.shape
				if (temp_N1,temp_N2) != (N1,N2):
					print('Error: file ' + gp2_files[f] + ' has dimension ' + temp_N1 + 'x' + temp_N2 + ' and does not match the one from file ' + gp2_files[0])
					return
				if(self.masking):
					temp[mask == 0] = 0
				Y[:,:,f] = temp
			PVAL, ADJ, NULL = compute_nbs(X,Y,self.NBS_thresh, self.NBS_permutations ,self.NBS_tail)
			if not op.exists(op.join(self.study_dir,'Results')):
				os.makedirs(op.join(self.study_dir,'Results'))
			np.savetxt(op.join(self.study_dir,'Results','NBS_thresh-'+ str(self.NBS_thresh) + '_perm-' + str(self.NBS_permutations) + '_tail-' + self.NBS_tail + '_pvalues.csv'),PVAL,delimiter=',')
			np.savetxt(op.join(self.study_dir,'Results','NBS_thresh-'+ str(self.NBS_thresh) + '_perm-' + str(self.NBS_permutations) + '_tail-' + self.NBS_tail + '_components.csv'),ADJ,delimiter=',')
			np.savetxt(op.join(self.study_dir,'Results','NBS_thresh-'+ str(self.NBS_thresh) + '_perm-' + str(self.NBS_permutations) + '_tail-' + self.NBS_tail + '_NULLsamples.csv'),NULL,delimiter=',')
			print('[  DONE  ]')

	except ImportError:
		print('Error: Numpy could not be loaded. NBS analysis is not available unless numpy is installed.')

    def _run_nodes_analysis_fired(self):
	print('[  NODE ANALYSIS  ]')
	node_measure_names = ['degree', 'betweenness', 'closeness', 'diameter']
	selected_node_measures = [self.degree,self.betweenness,self.closeness,self.diameter]
	if self.check_data(self.regress,self.paired,self.study_dir) and self.check_measures(selected_node_measures):
		robjects.globalenv['connectomeAnalyzerPath'] = os.getenv('ConnectomeAnalyzerPath')
		robjects.r('''
			source(\'''' + op.join(os.getenv('ConnectomeAnalyzerPath'),'LocalNodesAnalysis.r') + '''\')''')
		python_LocalNodesAnalysis = robjects.globalenv['LocalNodesAnalysis']
		
		for i in range(0,len(selected_node_measures)):
			if selected_node_measures[i]:
				python_LocalNodesAnalysis(self.study_dir,self.data_separ,False,self.mask_path,self.mask_separ,self.regress_separ,self.decomposition_separ,node_measure_names[i], self.binarize, self.is_weighted, self.decomposition_method, self.automatic_decompositions, self.decomposition_file, self.alpha1, self.alpha2, self.strong,self.alternative, self.MCPProcedure, self.test_type, self.plot_results,self.regression_file, self.regress, self.paired)
		print('[  DONE  ]')

    def _run_edges_analysis_fired(self):
	print('[  EDGE ANALYSIS  ]')
	if self.check_data(self.regress,self.paired,*self.multivariate_dirs):
		robjects.globalenv['connectomeAnalyzerPath'] = os.getenv('ConnectomeAnalyzerPath')
		robjects.r('''
			source(\'''' + op.join(os.getenv('ConnectomeAnalyzerPath'),'LocalConnectionAnalysis.r') + '''\')''')
		python_LocalConnectionAnalysis = robjects.globalenv['LocalConnectionAnalysis']
		python_LocalConnectionAnalysis(self.multivariate_dirs,self.data_separ,self.masking,self.mask_path,self.mask_separ,self.regress_separ,self.decomposition_separ,self.regression_file, self.regress,self.alpha1,self.alpha2,self.strong,self.test_type,self.n_perm, self.MCPProcedure,self.decomposition_method, self.automatic_decompositions, self.decomposition_file,self.paired,self.alternative,self.summary_function,self.plot_results,self.node_labels,self.nodes_color,self.edges_color)
		print('[  DONE  ]')


    def _run_single_subject_fired(self):
	print('[  SINGLE SUBJECT ANALYSIS  ]')
	selected_summary_statistics = [self.mean_degree, self.mean_betweenness, self.mean_closeness, self.mean_diameter, self.global_efficiency] 	# self.small_worldness
	summary_statistics_names = ['MeanDegree','MeanBetweenness','MeanCloseness','MeanDiameter','GlobalEfficiency']	# 'SmallWorldness'
	if self.check_file(self.single_matrix) and self.check_measures(selected_summary_statistics):
		robjects.globalenv['connectomeAnalyzerPath'] = os.getenv('ConnectomeAnalyzerPath')
		robjects.r('''
			source(\'''' + op.join(os.getenv('ConnectomeAnalyzerPath'),'singleSubjectAnalysis.r') + '''\')''')
		python_SingleSubjectAnalysis = robjects.globalenv['SingleSubjectAnalysis']
		for i in range(0,len(selected_summary_statistics)):
			if selected_summary_statistics[i]:
				python_SingleSubjectAnalysis(self.single_matrix,self.data_separ,summary_statistics_names[i],self.is_weighted,self.binarize)
		print('[  DONE  ]')
	
    def _run_relaxation_fired(self):
	print('[  P-VALUE RELAXATION  ]')
	if self.check_file(self.matrix_to_relax):
		robjects.globalenv['connectomeAnalyzerPath'] = os.getenv('ConnectomeAnalyzerPath')
		robjects.r('''
			source(\'''' + op.join(os.getenv('ConnectomeAnalyzerPath'),'relaxPvalues.r') + '''\')''')
		python_relaxPvalues = robjects.globalenv['relaxPvalues']
		python_relaxPvalues(self.matrix_to_relax,self.data_separ,self.decomposition_file,self.decomposition_separ,self.alpha1,self.alpha2,self.strong,self.MCPProcedure)
		print('[  DONE  ]')

    def save_state(self, cmaconfigfile):
        """ Save CMA Configuration state directly.
        Useful if you do not want to invoke the GUI
        
        Parameters
        ----------
        cmaconfigfile : string
            Absolute path and filename to store the CMA configuration
            pickled object
        
        """
        # check if path available
        if not os.path.exists(os.path.dirname(cmaconfigfile)):
            os.makedirs(os.path.abspath(os.path.dirname(cmaconfigfile)))
            
        import enthought.sweet_pickle as sp
        output = open(cmaconfigfile, 'wb')
        # Pickle the list using the highest protocol available.
        # copy object first
        tmpconf = CMAGUI()
        tmpconf.copy_traits(self)
        sp.dump(tmpconf, output, -1)
        output.close()

    def load_state(self, cmaconfigfile):
        """ Load CMA Configuration state directly.
        Useful if you do not want to invoke the GUI"""
        import enthought.sweet_pickle as sp        
        output = open(cmaconfigfile, 'rb')
        data = sp.load(output)
        self.__setstate__(data.__getstate__())
        output.close()

        
    def show(self):
        """ Show the GUI """
        self.configure_traits()
